#include <iostream>
#include <algorithm>
#include <vector>
#include <utility>

using namespace std;

// Templates from KACTL
#define rep(i, a, b) for(int i = a; i < (b); ++i)
#define trav(a, x) for(auto& a : x)
#define all(x) begin(x), end(x)
#define sz(x) (int)(x).size()
typedef long long ll;
typedef pair<int, int> pii;
typedef vector<int> vi;

struct Dinic {
	struct Edge {
		int to, rev;
		ll c, f;
	};
	vi lvl, ptr, q;
	vector<vector<Edge>> adj;
	Dinic(int n) : lvl(n), ptr(n), q(n), adj(n) {}
	void addEdge(int a, int b, ll c, int rcap = 0) {
		adj[a].push_back({b, sz(adj[b]), c, 0});
		adj[b].push_back({a, sz(adj[a]) - 1, rcap, 0});
	}
	ll dfs(int v, int t, ll f) {
		if (v == t || !f) return f;
		for (int& i = ptr[v]; i < sz(adj[v]); i++) {
			Edge& e = adj[v][i];
			if (lvl[e.to] == lvl[v] + 1)
				if (ll p = dfs(e.to, t, min(f, e.c - e.f))) {
					e.f += p, adj[e.to][e.rev].f -= p;
					return p;
				}
		}
		return 0;
	}
	ll calc(int s, int t) {
		ll flow = 0; q[0] = s;
        int L = 30;
		do { // 'int L=30' maybe faster for random data
			lvl = ptr = vi(sz(q));
			int qi = 0, qe = lvl[s] = 1;
			while (qi < qe && !lvl[t]) {
				int v = q[qi++];
				trav(e, adj[v])
					if (!lvl[e.to] && (e.c - e.f) >> (30 - L))
						q[qe++] = e.to, lvl[e.to] = lvl[v] + 1;
			}
			while (ll p = dfs(s, t, 1e18)) flow += p;
		} while (lvl[t]);
		return flow;
	}
};

int main() {
	cin.sync_with_stdio(0); cin.tie(0);
	cin.exceptions(cin.failbit);

    int n, p, q;
    ll s;
    cin >> n >> p >> q >> s;
    vi a(p), b(q);
    for (auto& x : a)
        cin >> x;
    for (auto& y : b)
        cin >> y;

    sort(all(a));
    sort(all(b));

    if (p < q) {
        swap(a, b);
        swap(p, q);
    }

    constexpr int INF = 1e9 + 7;

    int lo = -1;
    int hi = INF;
    while (lo + 1 < hi) {
        int mid = lo + (hi - lo) / 2;

        // normal left, seg tree right, src, sink
        Dinic d(p + q + q + 2);
        int src = p + q + q;
        int sink = src + 1;
        for (int i = 0; i < p; ++i) {
            d.addEdge(src, i, 1);
            ll lo_val = a[i] - mid;
            ll hi_val = min(s - a[i], 1LL * a[i] + mid);
            int x = lower_bound(all(b), lo_val) - begin(b);
            int y = upper_bound(all(b), hi_val) - begin(b);
            for (x += q, y += q; x < y; x /= 2, y /= 2) {
                if (x % 2) {
                    d.addEdge(i, p + x, 1);
                    x++;
                }
                if (y % 2) {
                    --y;
                    d.addEdge(i, p + y, q);
                }
            }
        }

        for (int i = 0; i < q; ++i) {
            for (int j = i + q; j /= 2;) {
                if (2 * j < q + q)
                    d.addEdge(p + j, p + 2 * j, q);
                if (2 * j + 1 < q + q)
                    d.addEdge(p + j, p + 2 * j + 1, q);
            }
        }

        for (int j = 0; j < q; ++j) {
            d.addEdge(p + q + j, sink, 1);
        }

        if (d.calc(src, sink) >= n)
            hi = mid;
        else
            lo = mid;
    }

    if (hi == INF)
        hi = -1;
    cout << hi << '\n';
}
